@@ -7,7 +7,7 @@ import re
 import smtplib
 import time
 
-from aprsd import messaging, threads
+from aprsd import messaging, stats, threads, trace
 import imapclient
 from validate_email import validate_email
 
@@ -17,58 +17,88 @@ LOG = logging.getLogger("APRSD")
 CONFIG = None
 
 
+@trace.trace
 def _imap_connect():
-    imap_port = CONFIG["imap"].get("port", 143)
-    use_ssl = CONFIG["imap"].get("use_ssl", False)
-    host = CONFIG["imap"]["host"]
+    imap_port = CONFIG["aprsd"]["email"]["imap"].get("port", 143)
+    use_ssl = CONFIG["aprsd"]["email"]["imap"].get("use_ssl", False)
+    host = CONFIG["aprsd"]["email"]["imap"]["host"]
     msg = "{}{}:{}".format("TLS " if use_ssl else "", host, imap_port)
     #    LOG.debug("Connect to IMAP host {} with user '{}'".
     #              format(msg, CONFIG['imap']['login']))
 
     try:
         server = imapclient.IMAPClient(
-            CONFIG["imap"]["host"],
+            CONFIG["aprsd"]["email"]["imap"]["host"],
             port=imap_port,
             use_uid=True,
             ssl=use_ssl,
+            timeout=30,
         )
-    except Exception:
-        LOG.error("Failed to connect IMAP server")
+    except Exception as e:
+        LOG.error("Failed to connect IMAP server", e)
         return
 
     try:
-        server.login(CONFIG["imap"]["login"], CONFIG["imap"]["password"])
+        server.login(
+            CONFIG["aprsd"]["email"]["imap"]["login"],
+            CONFIG["aprsd"]["email"]["imap"]["password"],
+        )
     except (imaplib.IMAP4.error, Exception) as e:
         msg = getattr(e, "message", repr(e))
         LOG.error("Failed to login {}".format(msg))
         return
 
     server.select_folder("INBOX")
+
+    server.fetch = trace.trace(server.fetch)
+    server.search = trace.trace(server.search)
+    server.remove_flags = trace.trace(server.remove_flags)
+    server.add_flags = trace.trace(server.add_flags)
     return server
 
 
+@trace.trace
 def _smtp_connect():
-    host = CONFIG["smtp"]["host"]
-    smtp_port = CONFIG["smtp"]["port"]
-    use_ssl = CONFIG["smtp"].get("use_ssl", False)
+    host = CONFIG["aprsd"]["email"]["smtp"]["host"]
+    smtp_port = CONFIG["aprsd"]["email"]["smtp"]["port"]
+    use_ssl = CONFIG["aprsd"]["email"]["smtp"].get("use_ssl", False)
     msg = "{}{}:{}".format("SSL " if use_ssl else "", host, smtp_port)
     LOG.debug(
-        "Connect to SMTP host {} with user '{}'".format(msg, CONFIG["imap"]["login"]),
+        "Connect to SMTP host {} with user '{}'".format(
+            msg,
+            CONFIG["aprsd"]["email"]["imap"]["login"],
+        ),
     )
 
     try:
         if use_ssl:
-            server = smtplib.SMTP_SSL(host=host, port=smtp_port)
+            server = smtplib.SMTP_SSL(
+                host=host,
+                port=smtp_port,
+                timeout=30,
+            )
         else:
-            server = smtplib.SMTP(host=host, port=smtp_port)
+            server = smtplib.SMTP(
+                host=host,
+                port=smtp_port,
+                timeout=30,
+            )
     except Exception:
         LOG.error("Couldn't connect to SMTP Server")
         return
 
     LOG.debug("Connected to smtp host {}".format(msg))
 
+    debug = CONFIG["aprsd"]["email"]["smtp"].get("debug", False)
+    if debug:
+        server.set_debuglevel(5)
+        server.sendmail = trace.trace(server.sendmail)
+
     try:
-        server.login(CONFIG["smtp"]["login"], CONFIG["smtp"]["password"])
+        server.login(
+            CONFIG["aprsd"]["email"]["smtp"]["login"],
+            CONFIG["aprsd"]["email"]["smtp"]["password"],
+        )
     except Exception:
         LOG.error("Couldn't connect to SMTP Server")
         return
@@ -78,7 +108,7 @@ def _smtp_connect():
 
 
 def validate_shortcuts(config):
-    shortcuts = config.get("shortcuts", None)
+    shortcuts = config["aprsd"]["email"].get("shortcuts", None)
     if not shortcuts:
         return
 
@@ -93,8 +123,8 @@ def validate_shortcuts(config):
             email_address=shortcuts[key],
             check_regex=True,
             check_mx=False,
-            from_address=config["smtp"]["login"],
-            helo_host=config["smtp"]["host"],
+            from_address=config["aprsd"]["email"]["smtp"]["login"],
+            helo_host=config["aprsd"]["email"]["smtp"]["host"],
             smtp_timeout=10,
             dns_timeout=10,
             use_blacklist=True,
@@ -109,14 +139,14 @@ def validate_shortcuts(config):
             delete_keys.append(key)
 
     for key in delete_keys:
-        del config["shortcuts"][key]
+        del config["aprsd"]["email"]["shortcuts"][key]
 
-    LOG.info("Available shortcuts: {}".format(config["shortcuts"]))
+    LOG.info("Available shortcuts: {}".format(config["aprsd"]["email"]["shortcuts"]))
 
 
 def get_email_from_shortcut(addr):
-    if CONFIG.get("shortcuts", False):
-        return CONFIG["shortcuts"].get(addr, addr)
+    if CONFIG["aprsd"]["email"].get("shortcuts", False):
+        return CONFIG["aprsd"]["email"]["shortcuts"].get(addr, addr)
     else:
         return addr
 
@@ -143,6 +173,7 @@ def validate_email_config(config, disable_validation=False):
         return False
 
 
+@trace.trace
 def parse_email(msgid, data, server):
     envelope = data[b"ENVELOPE"]
     # email address match
@@ -153,7 +184,12 @@ def parse_email(msgid, data, server):
     else:
         from_addr = "noaddr"
     LOG.debug("Got a message from '{}'".format(from_addr))
-    m = server.fetch([msgid], ["RFC822"])
+    try:
+        m = server.fetch([msgid], ["RFC822"])
+    except Exception as e:
+        LOG.exception("Couldn't fetch email from server in parse_email", e)
+        return
+
     msg = email.message_from_string(m[msgid][b"RFC822"].decode(errors="ignore"))
     if msg.is_multipart():
         text = ""
@@ -229,10 +265,11 @@ def parse_email(msgid, data, server):
 # end parse_email
 
 
+@trace.trace
 def send_email(to_addr, content):
     global check_email_delay
 
-    shortcuts = CONFIG["shortcuts"]
+    shortcuts = CONFIG["aprsd"]["email"]["shortcuts"]
     email_address = get_email_from_shortcut(to_addr)
     LOG.info("Sending Email_________________")
 
@@ -250,12 +287,17 @@ def send_email(to_addr, content):
 
     msg = MIMEText(content)
     msg["Subject"] = subject
-    msg["From"] = CONFIG["smtp"]["login"]
+    msg["From"] = CONFIG["aprsd"]["email"]["smtp"]["login"]
     msg["To"] = to_addr
     server = _smtp_connect()
     if server:
         try:
-            server.sendmail(CONFIG["smtp"]["login"], [to_addr], msg.as_string())
+            server.sendmail(
+                CONFIG["aprsd"]["email"]["smtp"]["login"],
+                [to_addr],
+                msg.as_string(),
+            )
+            stats.APRSDStats().email_tx_inc()
         except Exception as e:
             msg = getattr(e, "message", repr(e))
             LOG.error("Sendmail Error!!!! '{}'", msg)
@@ -268,6 +310,7 @@ def send_email(to_addr, content):
 # end send_email
 
 
+@trace.trace
 def resend_email(count, fromcall):
     global check_email_delay
     date = datetime.datetime.now()
@@ -276,7 +319,7 @@ def resend_email(count, fromcall):
     year = date.year
     today = "{}-{}-{}".format(day, month, year)
 
-    shortcuts = CONFIG["shortcuts"]
+    shortcuts = CONFIG["aprsd"]["email"]["shortcuts"]
     # swap key/value
     shortcuts_inverted = {v: k for k, v in shortcuts.items()}
 
@@ -286,7 +329,12 @@ def resend_email(count, fromcall):
         LOG.exception("Failed to Connect to IMAP. Cannot resend email ", e)
         return
 
-    messages = server.search(["SINCE", today])
+    try:
+        messages = server.search(["SINCE", today])
+    except Exception as e:
+        LOG.exception("Couldn't search for emails in resend_email ", e)
+        return
+
     # LOG.debug("%d messages received today" % len(messages))
 
     msgexists = False
@@ -294,18 +342,32 @@ def resend_email(count, fromcall):
     messages.sort(reverse=True)
     del messages[int(count) :]  # only the latest "count" messages
     for message in messages:
-        for msgid, data in list(server.fetch(message, ["ENVELOPE"]).items()):
+        try:
+            parts = server.fetch(message, ["ENVELOPE"]).items()
+        except Exception as e:
+            LOG.exception("Couldn't fetch email parts in resend_email", e)
+            continue
+
+        for msgid, data in list(parts):
             # one at a time, otherwise order is random
             (body, from_addr) = parse_email(msgid, data, server)
             # unset seen flag, will stay bold in email client
-            server.remove_flags(msgid, [imapclient.SEEN])
+            try:
+                server.remove_flags(msgid, [imapclient.SEEN])
+            except Exception as e:
+                LOG.exception("Failed to remove SEEN flag in resend_email", e)
+
             if from_addr in shortcuts_inverted:
                 # reverse lookup of a shortcut
                 from_addr = shortcuts_inverted[from_addr]
             # asterisk indicates a resend
             reply = "-" + from_addr + " * " + body.decode(errors="ignore")
             # messaging.send_message(fromcall, reply)
-            msg = messaging.TextMessage(CONFIG["aprs"]["login"], fromcall, reply)
+            msg = messaging.TextMessage(
+                CONFIG["aprs"]["login"],
+                fromcall,
+                reply,
+            )
             msg.send()
             msgexists = True
 
@@ -340,6 +402,7 @@ class APRSDEmailThread(threads.APRSDThread):
         self.msg_queues = msg_queues
         self.config = config
 
+    @trace.trace
     def run(self):
         global check_email_delay
 
@@ -349,6 +412,7 @@ class APRSDEmailThread(threads.APRSDThread):
         past = datetime.datetime.now()
         while not self.thread_stop:
             time.sleep(5)
+            stats.APRSDStats().email_thread_update()
             # always sleep for 5 seconds and see if we need to check email
             # This allows CTRL-C to stop the execution of this loop sooner
             # than check_email_delay time
@@ -362,7 +426,7 @@ class APRSDEmailThread(threads.APRSDThread):
                     check_email_delay += 1
                 LOG.debug("check_email_delay is " + str(check_email_delay) + " seconds")
 
-                shortcuts = CONFIG["shortcuts"]
+                shortcuts = CONFIG["aprsd"]["email"]["shortcuts"]
                 # swap key/value
                 shortcuts_inverted = {v: k for k, v in shortcuts.items()}
 
@@ -376,17 +440,30 @@ class APRSDEmailThread(threads.APRSDThread):
                 try:
                     server = _imap_connect()
                 except Exception as e:
-                    LOG.exception("Failed to get IMAP server Can't check email.", e)
+                    LOG.exception("IMAP failed to connect.", e)
 
                 if not server:
                     continue
 
-                messages = server.search(["SINCE", today])
+                try:
+                    messages = server.search(["SINCE", today])
+                except Exception as e:
+                    LOG.exception("IMAP failed to search for messages since today.", e)
+                    continue
                 LOG.debug("{} messages received today".format(len(messages)))
 
-                for msgid, data in server.fetch(messages, ["ENVELOPE"]).items():
+                try:
+                    _msgs = server.fetch(messages, ["ENVELOPE"])
+                except Exception as e:
+                    LOG.exception("IMAP failed to fetch/flag messages: ", e)
+                    continue
+
+                for msgid, data in _msgs.items():
                     envelope = data[b"ENVELOPE"]
-                    # LOG.debug('ID:%d  "%s" (%s)' % (msgid, envelope.subject.decode(), envelope.date))
+                    LOG.debug(
+                        'ID:%d  "%s" (%s)'
+                        % (msgid, envelope.subject.decode(), envelope.date),
+                    )
                     f = re.search(
                         r"'([[A-a][0-9]_-]+@[[A-a][0-9]_-\.]+)",
                         str(envelope.from_[0]),
@@ -399,16 +476,31 @@ class APRSDEmailThread(threads.APRSDThread):
                     # LOG.debug("Message flags/tags:  " + str(server.get_flags(msgid)[msgid]))
                     # if "APRS" not in server.get_flags(msgid)[msgid]:
                     # in python3, imap tags are unicode.  in py2 they're strings. so .decode them to handle both
-                    taglist = [
-                        x.decode(errors="ignore")
-                        for x in server.get_flags(msgid)[msgid]
-                    ]
+                    try:
+                        taglist = [
+                            x.decode(errors="ignore")
+                            for x in server.get_flags(msgid)[msgid]
+                        ]
+                    except Exception as e:
+                        LOG.exception("Failed to get flags.", e)
+                        break
+
                     if "APRS" not in taglist:
                         # if msg not flagged as sent via aprs
-                        server.fetch([msgid], ["RFC822"])
+                        try:
+                            server.fetch([msgid], ["RFC822"])
+                        except Exception as e:
+                            LOG.exception("Failed single server fetch for RFC822", e)
+                            break
+
                         (body, from_addr) = parse_email(msgid, data, server)
                         # unset seen flag, will stay bold in email client
-                        server.remove_flags(msgid, [imapclient.SEEN])
+                        try:
+                            server.remove_flags(msgid, [imapclient.SEEN])
+                        except Exception as e:
+                            LOG.exception("Failed to remove flags SEEN", e)
+                            # Not much we can do here, so lets try and
+                            # send the aprs message anyway
 
                         if from_addr in shortcuts_inverted:
                             # reverse lookup of a shortcut
@@ -422,14 +514,28 @@ class APRSDEmailThread(threads.APRSDThread):
                         )
                         self.msg_queues["tx"].put(msg)
                         # flag message as sent via aprs
-                        server.add_flags(msgid, ["APRS"])
-                        # unset seen flag, will stay bold in email client
-                        server.remove_flags(msgid, [imapclient.SEEN])
+                        try:
+                            server.add_flags(msgid, ["APRS"])
+                            # unset seen flag, will stay bold in email client
+                        except Exception as e:
+                            LOG.exception("Couldn't add APRS flag to email", e)
+
+                        try:
+                            server.remove_flags(msgid, [imapclient.SEEN])
+                        except Exception as e:
+                            LOG.exception("Couldn't remove seen flag from email", e)
+
                         # check email more often since we just received an email
                         check_email_delay = 60
+
                 # reset clock
+                LOG.debug("Done looping over Server.fetch, logging out.")
                 past = datetime.datetime.now()
-                server.logout()
+                try:
+                    server.logout()
+                except Exception as e:
+                    LOG.exception("IMAP failed to logout: ", e)
+                    continue
             else:
                 # We haven't hit the email delay yet.
                 # LOG.debug("Delta({}) < {}".format(now - past, check_email_delay))
@@ -438,6 +544,3 @@ class APRSDEmailThread(threads.APRSDThread):
         # Remove ourselves from the global threads list
         threads.APRSDThreadList().remove(self)
         LOG.info("Exiting")
-
-
-# end check_email()
